@@ -2,7 +2,7 @@
 #
 # Authors: Tom Kralidis <tomkralidis@gmail.com>
 #
-# Copyright (c) 2019 Tom Kralidis
+# Copyright (c) 2020 Tom Kralidis
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation
@@ -33,6 +33,7 @@ Returns content from plugins and sets reponses
 from datetime import datetime
 import json
 import logging
+import os
 import urllib.parse
 
 from dateutil.parser import parse as dateparse
@@ -43,9 +44,11 @@ from pygeoapi.linked_data import (geojson2geojsonld, jsonldify,
                                   jsonldify_collection)
 from pygeoapi.log import setup_logger
 from pygeoapi.plugin import load_plugin, PLUGINS
-from pygeoapi.provider.base import ProviderConnectionError, ProviderQueryError
-from pygeoapi.util import (dategetter, json_serial, render_j2_template,
-                           str2bool, TEMPLATES)
+from pygeoapi.provider.base import (
+    ProviderGenericError, ProviderConnectionError, ProviderNotFoundError,
+    ProviderQueryError, ProviderItemNotFoundError)
+from pygeoapi.util import (dategetter, filter_dict_by_key_value, json_serial,
+                           render_j2_template, str2bool, TEMPLATES)
 
 LOGGER = logging.getLogger(__name__)
 
@@ -89,7 +92,7 @@ def pre_process(func):
     return inner
 
 
-class API(object):
+class API:
     """API object"""
 
     def __init__(self, config):
@@ -263,7 +266,7 @@ class API(object):
     @jsonldify
     def describe_collections(self, headers_, format_, dataset=None):
         """
-        Provide feature collection metadata
+        Provide collection metadata
 
         :param headers_: copy of HEADERS object
         :param format_: format of requests,
@@ -286,21 +289,22 @@ class API(object):
             'links': []
         }
 
-        if all([dataset is not None,
-                dataset not in self.config['datasets'].keys()]):
+        collections = filter_dict_by_key_value(self.config['resources'],
+                                               'type', 'collection')
 
+        if all([dataset is not None, dataset not in collections.keys()]):
             exception = {
                 'code': 'InvalidParameterValue',
-                'description': 'Invalid feature collection'
+                'description': 'Invalid collection'
             }
             LOGGER.error(exception)
             return headers_, 400, json.dumps(exception)
 
         LOGGER.debug('Creating collections')
-        for k, v in self.config['datasets'].items():
+        for k, v in collections.items():
             collection = {'links': []}
             collection['id'] = k
-            collection['itemType'] = 'feature'
+            collection['itemType'] = 'Feature'
             collection['title'] = v['title']
             collection['description'] = v['description']
             collection['keywords'] = v['keywords']
@@ -342,24 +346,39 @@ class API(object):
                 collection['links'].append(lnk)
 
             LOGGER.debug('Adding JSON and HTML link relations')
+            collection['links'].append({
+                'type': 'application/json',
+                'rel': 'queryables',
+                'title': 'Queryables for this collection as JSON',
+                'href': '{}/collections/{}/queryables?f=json'.format(
+                    self.config['server']['url'], k)
+            })
+            collection['links'].append({
+                'type': 'text/html',
+                'rel': 'queryables',
+                'title': 'Queryables for this collection as HTML',
+                'href': '{}/collections/{}/queryables?f=html'.format(
+                    self.config['server']['url'], k)
+            })
+
             collection['links'].append({
                 'type': 'application/geo+json',
                 'rel': 'items',
-                'title': 'Features as GeoJSON',
+                'title': 'items as GeoJSON',
                 'href': '{}/collections/{}/items?f=json'.format(
                     self.config['server']['url'], k)
             })
             collection['links'].append({
                 'type': 'application/ld+json',
                 'rel': 'items',
-                'title': 'Features as RDF (GeoJSON-LD)',
+                'title': 'items as RDF (GeoJSON-LD)',
                 'href': '{}/collections/{}/items?f=jsonld'.format(
                     self.config['server']['url'], k)
             })
             collection['links'].append({
                 'type': 'text/html',
                 'rel': 'items',
-                'title': 'Features as HTML',
+                'title': 'Items as HTML',
                 'href': '{}/collections/{}/items?f=html'.format(
                     self.config['server']['url'], k)
             })
@@ -445,9 +464,89 @@ class API(object):
 
         return headers_, 200, json.dumps(fcm, default=json_serial)
 
+    @pre_process
+    @jsonldify
+    def get_collection_queryables(self, headers_, format_, dataset=None):
+        """
+        Provide collection queryables
+
+        :param headers_: copy of HEADERS object
+        :param format_: format of requests,
+                        pre checked by pre_process decorator
+        :param dataset: name of collection
+
+        :returns: tuple of headers, status code, content
+        """
+
+        if format_ is not None and format_ not in FORMATS:
+            exception = {
+                'code': 'InvalidParameterValue',
+                'description': 'Invalid format'
+            }
+            LOGGER.error(exception)
+            return headers_, 400, json.dumps(exception)
+
+        if any([dataset is None,
+                dataset not in self.config['resources'].keys()]):
+
+            exception = {
+                'code': 'InvalidParameterValue',
+                'description': 'Invalid collection'
+            }
+            LOGGER.error(exception)
+            return headers_, 400, json.dumps(exception)
+
+        LOGGER.debug('Creating collection queryables')
+        LOGGER.debug('Loading provider')
+        try:
+            p = load_plugin('provider',
+                            self.config['resources'][dataset]['provider'])
+        except ProviderConnectionError:
+            exception = {
+                'code': 'NoApplicableCode',
+                'description': 'connection error (check logs)'
+            }
+            LOGGER.error(exception)
+            return headers_, 500, json.dumps(exception)
+        except ProviderQueryError:
+            exception = {
+                'code': 'NoApplicableCode',
+                'description': 'query error (check logs)'
+            }
+            LOGGER.error(exception)
+            return headers_, 500, json.dumps(exception)
+
+        queryables = {
+            'queryables': []
+        }
+
+        for k, v in p.fields.items():
+            show_field = False
+            if p.properties:
+                if k in p.properties:
+                    show_field = True
+            else:
+                show_field = True
+
+            if show_field:
+                queryables['queryables'].append({
+                    'queryable': k,
+                    'type': v
+                })
+
+        if format_ == 'html':  # render
+            queryables['title'] = self.config['resources'][dataset]['title']
+            headers_['Content-Type'] = 'text/html'
+            content = render_j2_template(self.config, 'queryables.html',
+                                         queryables)
+
+            return headers_, 200, content
+
+        return headers_, 200, json.dumps(queryables, default=json_serial)
+
     def get_collection_items(self, headers, args, dataset, pathinfo=None):
         """
-        Queries feature collection
+        Queries collection
 
         :param headers: dict of HTTP headers
         :param args: dict of HTTP request parameters
@@ -461,14 +560,17 @@ class API(object):
 
         properties = []
         reserved_fieldnames = ['bbox', 'f', 'limit', 'startindex',
-                               'resulttype', 'datetime']
+                               'resulttype', 'datetime', 'sortby']
         formats = FORMATS
         formats.extend(f.lower() for f in PLUGINS['formatter'].keys())
 
-        if dataset not in self.config['datasets'].keys():
+        collections = filter_dict_by_key_value(self.config['resources'],
+                                               'type', 'collection')
+
+        if dataset not in collections.keys():
             exception = {
                 'code': 'InvalidParameterValue',
-                'description': 'Invalid feature collection'
+                'description': 'Invalid collection'
             }
             LOGGER.error(exception)
             return headers_, 400, json.dumps(exception, default=json_serial)
@@ -567,12 +669,12 @@ class API(object):
         datetime_invalid = False
 
         if (datetime_ is not None and
-                'temporal' in self.config['datasets'][dataset]['extents']):
-            te = self.config['datasets'][dataset]['extents']['temporal']
+                'temporal' in collections[dataset]['extents']):
+            te = collections[dataset]['extents']['temporal']
 
-            if te['begin'].tzinfo is None:
+            if te['begin'] is not None and te['begin'].tzinfo is None:
                 te['begin'] = te['begin'].replace(tzinfo=pytz.UTC)
-            if te['end'].tzinfo is None:
+            if te['end'] is not None and te['end'].tzinfo is None:
                 te['end'] = te['end'].replace(tzinfo=pytz.UTC)
 
             if '/' in datetime_:  # envelope
@@ -622,7 +724,7 @@ class API(object):
         LOGGER.debug('Loading provider')
         try:
             p = load_plugin('provider',
-                            self.config['datasets'][dataset]['provider'])
+                            collections[dataset]['provider'])
         except ProviderConnectionError:
             exception = {
                 'code': 'NoApplicableCode',
@@ -692,19 +794,26 @@ class API(object):
                               resulttype=resulttype, bbox=bbox,
                               datetime=datetime_, properties=properties,
                               sortby=sortby)
-        except ProviderConnectionError:
+        except ProviderConnectionError as err:
             exception = {
                 'code': 'NoApplicableCode',
                 'description': 'connection error (check logs)'
             }
-            LOGGER.error(exception)
+            LOGGER.error(err)
             return headers_, 500, json.dumps(exception)
-        except ProviderQueryError:
+        except ProviderQueryError as err:
             exception = {
                 'code': 'NoApplicableCode',
                 'description': 'query error (check logs)'
             }
-            LOGGER.error(exception)
+            LOGGER.error(err)
+            return headers_, 500, json.dumps(exception)
+        except ProviderGenericError as err:
+            exception = {
+                'code': 'NoApplicableCode',
+                'description': 'generic error (check logs)'
+            }
+            LOGGER.error(err)
             return headers_, 500, json.dumps(exception)
 
         serialized_query_params = ''
@@ -764,7 +873,7 @@ class API(object):
         content['links'].append(
             {
                 'type': 'application/json',
-                'title': self.config['datasets'][dataset]['title'],
+                'title': collections[dataset]['title'],
                 'rel': 'collection',
                 'href': '{}/collections/{}'.format(
                     self.config['server']['url'], dataset)
@@ -801,7 +910,7 @@ class API(object):
                 data=content,
                 options={
                     'provider_def':
-                        self.config['datasets'][dataset]['provider']
+                        collections[dataset]['provider']
                 }
             )
 
@@ -822,13 +931,13 @@ class API(object):
     @pre_process
     def get_collection_item(self, headers_, format_, dataset, identifier):
         """
-        Get a single feature
+        Get a single collection item
 
         :param headers_: copy of HEADERS object
         :param format_: format of requests,
                         pre checked by pre_process decorator
         :param dataset: dataset name
-        :param identifier: feature identifier
+        :param identifier: item identifier
 
         :returns: tuple of headers, status code, content
         """
@@ -843,20 +952,51 @@ class API(object):
 
         LOGGER.debug('Processing query parameters')
 
-        if dataset not in self.config['datasets'].keys():
+        collections = filter_dict_by_key_value(self.config['resources'],
+                                               'type', 'collection')
+
+        if dataset not in collections.keys():
             exception = {
                 'code': 'InvalidParameterValue',
-                'description': 'Invalid feature collection'
+                'description': 'Invalid collection'
             }
             LOGGER.error(exception)
             return headers_, 400, json.dumps(exception)
 
         LOGGER.debug('Loading provider')
-        p = load_plugin('provider',
-                        self.config['datasets'][dataset]['provider'])
+        p = load_plugin('provider', collections[dataset]['provider'])
 
-        LOGGER.debug('Fetching id {}'.format(identifier))
-        content = p.get(identifier)
+        try:
+            LOGGER.debug('Fetching id {}'.format(identifier))
+            content = p.get(identifier)
+        except ProviderConnectionError as err:
+            exception = {
+                'code': 'NoApplicableCode',
+                'description': 'connection error (check logs)'
+            }
+            LOGGER.error(err)
+            return headers_, 500, json.dumps(exception)
+        except ProviderItemNotFoundError:
+            exception = {
+                'code': 'NotFound',
+                'description': 'identifier not found'
+            }
+            LOGGER.error(exception)
+            return headers_, 404, json.dumps(exception)
+        except ProviderQueryError as err:
+            exception = {
+                'code': 'NoApplicableCode',
+                'description': 'query error (check logs)'
+            }
+            LOGGER.error(err)
+            return headers_, 500, json.dumps(exception)
+        except ProviderGenericError as err:
+            exception = {
+                'code': 'NoApplicableCode',
+                'description': 'generic error (check logs)'
+            }
+            LOGGER.error(err)
+            return headers_, 500, json.dumps(exception)
 
         if content is None:
             exception = {
@@ -887,7 +1027,7 @@ class API(object):
             }, {
             'rel': 'collection',
             'type': 'application/json',
-            'title': self.config['datasets'][dataset]['title'],
+            'title': collections[dataset]['title'],
             'href': '{}/collections/{}'.format(
                 self.config['server']['url'], dataset)
             }, {
@@ -906,7 +1046,7 @@ class API(object):
         if format_ == 'html':  # render
             headers_['Content-Type'] = 'text/html'
 
-            content['title'] = self.config['datasets'][dataset]['title']
+            content['title'] = collections[dataset]['title']
             content = render_j2_template(self.config, 'item.html',
                                          content)
             return headers_, 200, content
@@ -919,6 +1059,153 @@ class API(object):
 
         return headers_, 200, json.dumps(content, default=json_serial)
 
+    @pre_process
+    @jsonldify
+    def get_stac_root(self, headers_, format_):
+
+        if format_ is not None and format_ not in FORMATS:
+            exception = {
+                'code': 'InvalidParameterValue',
+                'description': 'Invalid format'
+            }
+            LOGGER.error(exception)
+            return headers_, 400, json.dumps(exception)
+
+        id_ = 'pygeoapi-stac'
+        stac_version = '0.6.2'
+        stac_url = os.path.join(self.config['server']['url'], 'stac')
+
+        content = {
+            'id': id_,
+            'stac_version': stac_version,
+            'title': self.config['metadata']['identification']['title'],
+            'description': self.config['metadata']['identification']['description'],  # noqa
+            'license': self.config['metadata']['license']['name'],
+            'providers': [{
+                'name': self.config['metadata']['provider']['name'],
+                'url': self.config['metadata']['provider']['url'],
+            }],
+            'links': []
+        }
+
+        stac_collections = filter_dict_by_key_value(self.config['resources'],
+                                                    'type', 'stac-collection')
+
+        for key, value in stac_collections.items():
+            content['links'].append({
+                'rel': 'collection',
+                'href': '{}/{}?f=json'.format(stac_url, key),
+                'type': 'application/json'
+            })
+            content['links'].append({
+                'rel': 'collection',
+                'href': '{}/{}'.format(stac_url, key),
+                'type': 'text/html'
+            })
+
+        if format_ == 'html':  # render
+            headers_['Content-Type'] = 'text/html'
+            content = render_j2_template(self.config, 'stac/root.html',
+                                         content)
+            return headers_, 200, content
+
+        return headers_, 200, json.dumps(content, default=json_serial)
+
+    @pre_process
+    @jsonldify
+    def get_stac_path(self, headers_, format_, path):
+
+        if format_ is not None and format_ not in FORMATS:
+            exception = {
+                'code': 'InvalidParameterValue',
+                'description': 'Invalid format'
+            }
+            LOGGER.error(exception)
+            return headers_, 400, json.dumps(exception)
+
+        LOGGER.debug('Path: {}'.format(path))
+        dir_tokens = path.split('/')
+        if dir_tokens:
+            dataset = dir_tokens[0]
+
+        stac_collections = filter_dict_by_key_value(self.config['resources'],
+                                                    'type', 'stac-collection')
+
+        if dataset not in stac_collections:
+            exception = {
+                'code': 'NotFound',
+                'description': 'collection not found'
+            }
+            LOGGER.error(exception)
+            return headers_, 404, json.dumps(exception)
+
+        LOGGER.debug('Loading provider')
+        try:
+            p = load_plugin('provider', stac_collections[dataset]['provider'])
+        except ProviderConnectionError as err:
+            LOGGER.error(err)
+            exception = {
+                'code': 'NoApplicableCode',
+                'description': 'connection error (check logs)'
+            }
+            LOGGER.error(exception)
+            return headers_, 500, json.dumps(exception)
+
+        id_ = '{}-stac'.format(dataset)
+        stac_version = '0.6.2'
+        description = stac_collections[dataset]['description']
+
+        content = {
+            'id': id_,
+            'stac_version': stac_version,
+            'description': description,
+            'links': []
+        }
+        try:
+            stac_data = p.get_data_path(
+                os.path.join(self.config['server']['url'], 'stac'),
+                path,
+                path.replace(dataset, '', 1)
+            )
+        except ProviderNotFoundError as err:
+            LOGGER.error(err)
+            exception = {
+                'code': 'NotFound',
+                'description': 'resource not found'
+            }
+            return headers_, 404, json.dumps(exception)
+        except Exception as err:
+            LOGGER.error(err)
+            exception = {
+                'code': 'NoApplicableCode',
+                'description': 'data query error'
+            }
+            return headers_, 500, json.dumps(exception)
+
+        if isinstance(stac_data, dict):
+            content.update(stac_data)
+            content['links'].extend(stac_collections[dataset]['links'])
+
+            if format_ == 'html':  # render
+                headers_['Content-Type'] = 'text/html'
+                content['path'] = path
+                if 'assets' in content:  # item view
+                    content = render_j2_template(self.config,
+                                                 'stac/item.html',
+                                                 content)
+                else:
+                    content = render_j2_template(self.config,
+                                                 'stac/catalog.html',
+                                                 content)
+
+                return headers_, 200, content
+
+            return headers_, 200, json.dumps(content, default=json_serial)
+
+        else:  # send back file
+            headers_.pop('Content-Type', None)
+            return headers_, 200, stac_data
+
     @pre_process
     @jsonldify
     def describe_processes(self, headers_, format_, process=None):
@@ -940,7 +1227,8 @@ class API(object):
             LOGGER.error(exception)
             return headers_, 400, json.dumps(exception)
 
-        processes_config = self.config.get('processes', {})
+        processes_config = filter_dict_by_key_value(self.config['resources'],
+                                                    'type', 'process')
 
         if processes_config:
             if process is not None:
@@ -1011,7 +1299,8 @@ class API(object):
             LOGGER.error(exception)
             return headers_, 400, json.dumps(exception)
 
-        processes = self.config.get('processes', {})
+        processes = filter_dict_by_key_value(self.config['resources'],
+                                             'type', 'process')
 
         if process not in processes:
             exception = {
