@@ -60,7 +60,7 @@ def jsonldify(func):
         provider = meta.get('provider', {})
         ident = meta.get('identification', {})
         fcmld = {
-          "@context": "https://schema.org",
+          "@context": "https://schema.org/docs/jsonldcontext.jsonld",
           "@type": "DataCatalog",
           "@id": cfg.get('server', {}).get('url', None),
           "url": cfg.get('server', {}).get('url', None),
@@ -131,7 +131,8 @@ def jsonldify_collection(cls, collection):
         "license": cls.fcmld['license'],
         "keywords": collection.get('keywords', None),
         "spatial": None if (not hascrs84 or not bbox) else [{
-            "Place": {
+            "@type": "Place",
+            "geo": {
                 "@type": "GeoShape",
                 "box": '{},{} {},{}'.format(*_bbox[0:2], *_bbox[2:4])
             }
@@ -173,7 +174,7 @@ def geojson2geojsonld(config, data, dataset, identifier=None):
 
     :returns: string of rendered JSON (GeoJSON-LD)
     """
-    context = config['datasets'][dataset].get('context', [])
+    context = config['resources'][dataset].get('context', [])
     data['id'] = (
         '{}/collections/{}/items/{}' if identifier
         else '{}/collections/{}/items'
