@@ -2,7 +2,7 @@
 #
 # Authors: Tom Kralidis <tomkralidis@gmail.com>
 #
-# Copyright (c) 2019 Tom Kralidis
+# Copyright (c) 2020 Tom Kralidis
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation
@@ -29,23 +29,37 @@
 
 from copy import deepcopy
 import logging
+import os
 
 import click
 import yaml
 
 from pygeoapi.plugin import load_plugin
-from pygeoapi.util import yaml_load
+from pygeoapi.util import filter_dict_by_key_value, yaml_load
 
 LOGGER = logging.getLogger(__name__)
 
-# TODO: handle this better once schemas are public/final
-# allow also for schema caching
 OPENAPI_YAML = {
     'oapif': 'http://schemas.opengis.net/ogcapi/features/part1/1.0/openapi/ogcapi-features-1.yaml',  # noqa
     'oapip': 'https://raw.githubusercontent.com/opengeospatial/wps-rest-binding/master/core/openapi'  # noqa
 }
 
 
+def get_ogc_schemas_location(server_config):
+
+    osl = server_config.get('ogc_schemas_location', None)
+
+    value = 'http://schemas.opengis.net'
+
+    if osl is not None:
+        if osl.startswith('http'):
+            value = osl
+        elif osl.startswith('/'):
+            value = os.path.join(server_config['url'], 'schemas')
+
+    return value
+
+
 # TODO: remove this function once OGC API - Processing is final
 def gen_media_type_object(media_type, api_type, path):
     """
@@ -101,6 +115,10 @@ def get_oas_30(cfg):
     """
 
     paths = {}
+
+    osl = get_ogc_schemas_location(cfg['server'])
+    OPENAPI_YAML['oapif'] = os.path.join(osl, 'ogcapi/features/part1/1.0/openapi/ogcapi-features-1.yaml')  # noqa
+
     LOGGER.debug('setting up server info')
     oas = {
         'openapi': '3.0.2',
@@ -180,8 +198,8 @@ def get_oas_30(cfg):
 
     paths['/collections'] = {
         'get': {
-            'summary': 'Feature Collections',
-            'description': 'Feature Collections',
+            'summary': 'Collections',
+            'description': 'Collections',
             'tags': ['server'],
             'parameters': [
                 {'$ref': '#/components/parameters/f'}
@@ -258,7 +276,10 @@ def get_oas_30(cfg):
     items_f['schema']['enum'].append('csv')
 
     LOGGER.debug('setting up datasets')
-    for k, v in cfg['datasets'].items():
+    collections = filter_dict_by_key_value(cfg['resources'],
+                                           'type', 'collection')
+
+    for k, v in collections.items():
         collection_name_path = '/collections/{}'.format(k)
         tag = {
             'name': k,
@@ -277,7 +298,7 @@ def get_oas_30(cfg):
 
         paths[collection_name_path] = {
             'get': {
-                'summary': 'Get feature collection metadata'.format(v['title']),  # noqa
+                'summary': 'Get collection metadata'.format(v['title']),  # noqa
                 'description': v['description'],
                 'tags': [k],
                 'parameters': [
@@ -296,7 +317,7 @@ def get_oas_30(cfg):
 
         paths[items_path] = {
             'get': {
-                'summary': 'Get {} features'.format(v['title']),
+                'summary': 'Get {} items'.format(v['title']),
                 'description': v['description'],
                 'tags': [k],
                 'parameters': [
@@ -315,48 +336,71 @@ def get_oas_30(cfg):
             }
         }
 
-        p = load_plugin('provider', cfg['datasets'][k]['provider'])
+        p = load_plugin('provider', collections[k]['provider'])
+
+        if p.fields:
+            queryables_path = '{}/queryables'.format(collection_name_path)
+
+            paths[queryables_path] = {
+                'get': {
+                    'summary': 'Get {} queryables'.format(v['title']),
+                    'description': v['description'],
+                    'tags': [k],
+                    'parameters': [
+                        items_f,
+                    ],
+                    'responses': {
+                        200: {'$ref': '{}#/components/responses/Features'.format(OPENAPI_YAML['oapif'])},  # noqa
+                        400: {'$ref': '{}#/components/responses/InvalidParameter'.format(OPENAPI_YAML['oapif'])},  # noqa
+                        404: {'$ref': '{}#/components/responses/NotFound'.format(OPENAPI_YAML['oapif'])},  # noqa
+                        500: {'$ref': '{}#/components/responses/ServerError'.format(OPENAPI_YAML['oapif'])}  # noqa
+                    }
+                }
+            }
 
         if p.time_field is not None:
             paths[items_path]['get']['parameters'].append(
                 {'$ref': '{}#/components/parameters/datetime'.format(OPENAPI_YAML['oapif'])})  # noqa
 
-        for k2, v2 in p.fields.items():
-            if p.properties and k2 in p.properties:
-                path_ = '{}/items'.format(collection_name_path)
+        for field, type in p.fields.items():
 
-                if v2['type'] == 'date':
-                    schema = {
-                        'type': 'string',
-                        'format': 'date'
-                    }
-                elif v2['type'] == 'float':
-                    schema = {
-                        'type': 'number',
-                        'format': 'float'
-                    }
-                elif v2['type'] == 'long':
-                    schema = {
-                        'type': 'integer',
-                        'format': 'int64'
-                    }
-                else:
-                    schema = {
-                        'type': v2['type']
-                    }
+            if p.properties and field not in p.properties:
+                LOGGER.debug('Provider specified not to advertise property')
+                continue
 
-                paths['{}'.format(path_)]['get']['parameters'].append({
-                    'name': k2,
-                    'in': 'query',
-                    'required': False,
-                    'schema': schema,
-                    'style': 'form',
-                    'explode': False
-                })
+            if type == 'date':
+                schema = {
+                    'type': 'string',
+                    'format': 'date'
+                }
+            elif type == 'float':
+                schema = {
+                    'type': 'number',
+                    'format': 'float'
+                }
+            elif type == 'long':
+                schema = {
+                    'type': 'integer',
+                    'format': 'int64'
+                }
+            else:
+                schema = {
+                    'type': type
+                }
+
+            path_ = '{}/items'.format(collection_name_path)
+            paths['{}'.format(path_)]['get']['parameters'].append({
+                'name': field,
+                'in': 'query',
+                'required': False,
+                'schema': schema,
+                'style': 'form',
+                'explode': False
+            })
 
         paths['{}/items/{{featureId}}'.format(collection_name_path)] = {
             'get': {
-                'summary': 'Get {} feature by id'.format(v['title']),
+                'summary': 'Get {} item by id'.format(v['title']),
                 'description': v['description'],
                 'tags': [k],
                 'parameters': [
@@ -372,6 +416,21 @@ def get_oas_30(cfg):
             }
         }
 
+    LOGGER.debug('setting up STAC')
+    paths['/stac'] = {
+        'get': {
+            'summary': 'SpatioTemporal Asset Catalog',
+            'description': 'SpatioTemporal Asset Catalog',
+            'tags': ['stac'],
+            'parameters': [],
+            'responses': {
+                200: {'$ref': '#/components/responses/200'},
+                'default': {'$ref': '#/components/responses/default'}
+            }
+        }
+    }
+
+    LOGGER.debug('setting up processes')
     paths['/processes'] = {
         'get': {
             'summary': 'Processes',
@@ -387,9 +446,7 @@ def get_oas_30(cfg):
         }
     }
 
-    LOGGER.debug('setting up processes')
-
-    processes = cfg.get('processes', {})
+    processes = filter_dict_by_key_value(cfg['resources'], 'type', 'process')
 
     if processes:
         for k, v in processes.items():
@@ -413,8 +470,7 @@ def get_oas_30(cfg):
 
             paths[process_name_path] = {
                 'get': {
-                    'summary': 'Get process metadata'.format(
-                        p.metadata['title']),
+                    'summary': 'Get process metadata',
                     'description': p.metadata['description'],
                     'tags': [k],
                     'parameters': [
