@@ -2,7 +2,7 @@
 #
 # Authors: Tom Kralidis <tomkralidis@gmail.com>
 #
-# Copyright (c) 2019 Tom Kralidis
+# Copyright (c) 2020 Tom Kralidis
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation
@@ -30,6 +30,7 @@
 
 import importlib
 import logging
+import os
 
 LOGGER = logging.getLogger(__name__)
 
@@ -40,11 +41,11 @@ PLUGINS = {
         'CSV': 'pygeoapi.provider.csv_.CSVProvider',
         'Elasticsearch': 'pygeoapi.provider.elasticsearch_.ElasticsearchProvider',  # noqa
         'GeoJSON': 'pygeoapi.provider.geojson.GeoJSONProvider',
-        'GeoPackage': 'pygeoapi.provider.geopackage.GeoPackageProvider',
         'OGR': 'pygeoapi.provider.ogr.OGRProvider',
         'PostgreSQL': 'pygeoapi.provider.postgresql.PostgreSQLProvider',
         'SQLiteGPKG': 'pygeoapi.provider.sqlite.SQLiteGPKGProvider',
-        'MongoDB': 'pygeoapi.provider.mongo.MongoProvider'
+        'MongoDB': 'pygeoapi.provider.mongo.MongoProvider',
+        'FileSystem': 'pygeoapi.provider.filesystem.FileSystemProvider'
     },
     'formatter': {
         'CSV': 'pygeoapi.formatter.csv_.CSVFormatter'
@@ -67,6 +68,10 @@ def load_plugin(plugin_type, plugin_def):
 
     name = plugin_def['name']
 
+    if name == "OGR":
+        os.environ["OGR_GEOJSON_MAX_OBJ_SIZE"] = os.environ.get(
+            "OGR_GEOJSON_MAX_OBJ_SIZE", "20MB")
+
     if plugin_type not in PLUGINS.keys():
         msg = 'Plugin type {} not found'.format(plugin_type)
         LOGGER.exception(msg)
