@@ -48,14 +48,14 @@ import json
 import psycopg2
 from psycopg2.sql import SQL, Identifier, Literal
 from pygeoapi.provider.base import BaseProvider, \
-    ProviderConnectionError, ProviderQueryError
+    ProviderConnectionError, ProviderQueryError, ProviderItemNotFoundError
 
 from psycopg2.extras import RealDictCursor
 
 LOGGER = logging.getLogger(__name__)
 
 
-class DatabaseConnection(object):
+class DatabaseConnection:
     """Database connection class to be used as 'with' statement.
      The class returns a connection object.
     """
@@ -101,6 +101,7 @@ class DatabaseConnection(object):
                 search_path={}'.format(",".join(search_path))
                 LOGGER.debug('Using search path: {} '.format(search_path))
             self.conn = psycopg2.connect(**self.conn_dic)
+            self.conn.set_client_encoding('utf8')
 
         except psycopg2.OperationalError:
             LOGGER.error("Couldn't connect to Postgis using:{}".format(
@@ -174,6 +175,35 @@ class PostgreSQLProvider(BaseProvider):
                 self.fields = db.fields
         return self.fields
 
+    def __get_where_clauses(self, properties=[], bbox=[]):
+        """
+        Generarates WHERE conditions to be implemented in query.
+        Private method mainly associated with query method
+        :param properties: list of tuples (name, value)
+        :param bbox: bounding box [minx,miny,maxx,maxy]
+
+        :returns: psycopg2.sql.Composed or psycopg2.sql.SQL
+        """
+
+        where_conditions = []
+        if properties:
+            property_clauses = [SQL('{} = {}').format(
+                Identifier(k), Literal(v)) for k, v in properties]
+            where_conditions += property_clauses
+        if bbox:
+            bbox_clause = SQL('{} && ST_MakeEnvelope({})').format(
+                Identifier(self.geom), SQL(', ').join(
+                    [Literal(bbox_coord) for bbox_coord in bbox]))
+            where_conditions.append(bbox_clause)
+
+        if where_conditions:
+            where_clause = SQL(' WHERE {}').format(
+                SQL(' AND ').join(where_conditions))
+        else:
+            where_clause = SQL('')
+
+        return where_clause
+
     def query(self, startindex=0, limit=10, resulttype='results',
               bbox=[], datetime=None, properties=[], sortby=[]):
         """
@@ -198,13 +228,16 @@ class PostgreSQLProvider(BaseProvider):
             with DatabaseConnection(self.conn_dic,
                                     self.table, context="hits") as db:
                 cursor = db.conn.cursor(cursor_factory=RealDictCursor)
-                sql_query = SQL("select count(*) as hits from {}").\
-                    format(Identifier(self.table))
+
+                where_clause = self.__get_where_clauses(
+                    properties=properties, bbox=bbox)
+                sql_query = SQL("SELECT COUNT(*) as hits from {} {}").\
+                    format(Identifier(self.table), where_clause)
                 try:
                     cursor.execute(sql_query)
                 except Exception as err:
                     LOGGER.error('Error executing sql_query: {}: {}'.format(
-                        sql_query.as_string(cursor)), err)
+                        sql_query.as_string(cursor), err))
                     raise ProviderQueryError()
 
                 hits = cursor.fetchone()["hits"]
@@ -215,29 +248,12 @@ class PostgreSQLProvider(BaseProvider):
 
         with DatabaseConnection(self.conn_dic, self.table) as db:
             cursor = db.conn.cursor(cursor_factory=RealDictCursor)
-            where_conditions = []
-            if properties:
-                property_clauses = \
-                    [SQL('{} = {}').format(
-                        Identifier(k), Literal(v)) for k, v in properties]
-                where_conditions += property_clauses
-            if bbox:
-                bbox_clause = SQL('{} && ST_MakeEnvelope({})').format(
-                    Identifier(self.geom),
-                    SQL(', ').join(
-                        [Literal(bbox_coord) for bbox_coord in bbox]
-                    )
-                )
-                where_conditions.append(bbox_clause)
 
-            if where_conditions:
-                where_clause = SQL(' WHERE {}').format(
-                    SQL(' AND ').join(where_conditions)
-                )
-            else:
-                where_clause = SQL('')
+            where_clause = self.__get_where_clauses(
+                properties=properties, bbox=bbox)
+
             sql_query = SQL("DECLARE \"geo_cursor\" CURSOR FOR \
-             SELECT {},ST_AsGeoJSON({}) FROM {}{}").\
+             SELECT DISTINCT {},ST_AsGeoJSON({}) FROM {}{}").\
                 format(db.columns,
                        Identifier(self.geom),
                        Identifier(self.table),
@@ -286,7 +302,7 @@ class PostgreSQLProvider(BaseProvider):
             Identifier(self.id_field),
         ), (identifier,))
         item = cursor.fetchall()
-        id_ = item[0]['id']
+        id_ = item[0]['id'] if item else identifier
         return id_
 
     def get_next(self, cursor, identifier):
@@ -305,7 +321,7 @@ class PostgreSQLProvider(BaseProvider):
             Identifier(self.id_field),
         ), (identifier,))
         item = cursor.fetchall()
-        id_ = item[0]['id']
+        id_ = item[0]['id'] if item else identifier
         return id_
 
     def get(self, identifier):
@@ -322,7 +338,7 @@ class PostgreSQLProvider(BaseProvider):
         with DatabaseConnection(self.conn_dic, self.table) as db:
             cursor = db.conn.cursor(cursor_factory=RealDictCursor)
 
-            sql_query = SQL("select {},ST_AsGeoJSON({}) \
+            sql_query = SQL("SELECT {},ST_AsGeoJSON({}) \
             from {} WHERE {}=%s").format(db.columns,
                                          Identifier(self.geom),
                                          Identifier(self.table),
@@ -338,12 +354,20 @@ class PostgreSQLProvider(BaseProvider):
                 LOGGER.error(err)
                 raise ProviderQueryError()
 
-            row_data = cursor.fetchall()[0]
+            results = cursor.fetchall()
+            row_data = None
+            if results:
+                row_data = results[0]
             feature = self.__response_feature(row_data)
 
-            feature['prev'] = self.get_previous(cursor, identifier)
-            feature['next'] = self.get_next(cursor, identifier)
-            return feature
+            if feature:
+                feature['prev'] = self.get_previous(cursor, identifier)
+                feature['next'] = self.get_next(cursor, identifier)
+                return feature
+            else:
+                err = 'item {} not found'.format(identifier)
+                LOGGER.error(err)
+                raise ProviderItemNotFoundError(err)
 
     def __response_feature(self, row_data):
         """
@@ -354,17 +378,20 @@ class PostgreSQLProvider(BaseProvider):
         :returns: `dict` of GeoJSON Feature
         """
 
-        rd = dict(row_data)
-        feature = {
-            'type': 'Feature'
-        }
-        feature["geometry"] = json.loads(
-            rd.pop('st_asgeojson'))
+        if row_data:
+            rd = dict(row_data)
+            feature = {
+                'type': 'Feature'
+            }
+            feature["geometry"] = json.loads(
+                rd.pop('st_asgeojson'))
 
-        feature['properties'] = rd
-        feature['id'] = feature['properties'].get(self.id_field)
+            feature['properties'] = rd
+            feature['id'] = feature['properties'].get(self.id_field)
 
-        return feature
+            return feature
+        else:
+            return None
 
     def __response_feature_hits(self, hits):
         """Assembles GeoJSON/Feature number
