@@ -2,7 +2,7 @@
 #
 # Authors: Tom Kralidis <tomkralidis@gmail.com>
 #
-# Copyright (c) 2019 Tom Kralidis
+# Copyright (c) 2020 Tom Kralidis
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation
@@ -33,6 +33,7 @@ from datetime import date, datetime, time
 from decimal import Decimal
 import json
 import logging
+import mimetypes
 import os
 import re
 from urllib.parse import urlparse
@@ -47,6 +48,9 @@ LOGGER = logging.getLogger(__name__)
 TEMPLATES = '{}{}templates'.format(os.path.dirname(
     os.path.realpath(__file__)), os.sep)
 
+mimetypes.add_type('text/plain', '.yaml')
+mimetypes.add_type('text/plain', '.yml')
+
 
 def dategetter(date_property, collection):
     """
@@ -149,6 +153,18 @@ def to_json(dict_):
     return json.dumps(dict_, default=json_serial)
 
 
+def get_path_basename(urlpath):
+    """
+    Helper function to derive file basename
+
+    :param urlpath: URL path
+
+    :returns: string of basename of URL path
+    """
+
+    return os.path.basename(urlpath)
+
+
 def json_serial(obj):
     """
     helper function to convert to JSON non-default
@@ -194,8 +210,71 @@ def render_j2_template(config, template, data):
     """
 
     env = Environment(loader=FileSystemLoader(TEMPLATES))
+
     env.filters['to_json'] = to_json
     env.globals.update(to_json=to_json)
 
+    env.filters['get_path_basename'] = get_path_basename
+    env.globals.update(get_path_basename=get_path_basename)
+
+    env.filters['get_breadcrumbs'] = get_breadcrumbs
+    env.globals.update(get_breadcrumbs=get_breadcrumbs)
+
+    env.filters['filter_dict_by_key_value'] = filter_dict_by_key_value
+    env.globals.update(filter_dict_by_key_value=filter_dict_by_key_value)
+
     template = env.get_template(template)
     return template.render(config=config, data=data, version=__version__)
+
+
+def get_mimetype(filename):
+    """
+    helper function to return MIME type of a given file
+
+    :param filename: filename (with extension)
+
+    :returns: MIME type of given filename
+    """
+
+    return mimetypes.guess_type(filename)[0]
+
+
+def get_breadcrumbs(urlpath):
+    """
+    helper function to make breadcrumbs from a URL path
+
+    :param urlpath: URL path
+
+    :returns: `list` of `dict` objects of labels and links
+    """
+
+    links = []
+
+    tokens = urlpath.split('/')
+
+    s = ''
+    for t in tokens:
+        if s:
+            s += '/' + t
+        else:
+            s = t
+        links.append({
+            'href': s,
+            'title': t,
+        })
+
+    return links
+
+
+def filter_dict_by_key_value(dict_, key, value):
+    """
+    helper generator function to filter a dict by a dict key
+
+    :param dict_: ``dict``
+    :param key: dict key
+    :param value: dict key value
+
+    :returns: filtered ``dict``
+    """
+
+    return {k: v for (k, v) in dict_.items() if v[key] == value}
